--!parent
--!encoding=utf-8

--****************************************************************************
-- @desc simple helper lib for button and menu creation
--****************************************************************************

property _sn, _x, _y, _marg, _dw, _props
property _tmp
property _menuField, _menuData

----------------------------------------
-- @constructor
----------------------------------------
on new (me, sn)

	_props = [:]
	_props["alignment"] = "center"
	_props["fontsize"] = 10
	_props["fontStyle"] = "bold"
	_props["font"] = "MS Sans Serif"
		
	_tmp = new(#field)
	cnt = count(_props)
	repeat with i = 1 to cnt
		do("_tmp."&_props.getPropAt(i)&"=_props[i]")
	end repeat
	
	_tmp.rect = rect(0,0,1024,0)

	if voidP(sn) then _sn = 1
	else _sn = sn

	-- button defaults
	_x = 10
	_y = 10
	_marg = 10
	_dw = 10

	_menuField = new(#field)

	return me
end

----------------------------------------
-- 
----------------------------------------
on buttonSetProps (me, x0, y0, marg, dw, props)
	_x = x0
	_y = y0
	_marg = marg
	_dw = dw	
	cnt = count(props)
	repeat with i = 1 to cnt
		_props[props.getPropAt(i)] = props[i]
		do("_tmp."&props.getPropAt(i)&"=props[i]")
	end repeat
end

----------------------------------------
-- creates a button sprite
----------------------------------------
on buttonCreate (me, txt, callback)
	_tmp.text = txt
	w = _tmp.charPosToLoc(txt.length+1)[1] + 2*_marg --16
	
	if _x + w>_movie.stage.rect.width then
		_x = 10
		_y = _y + 25
	end if
	
	m = new(#button)
	m.rect = rect(0,0,w,0)
	m.text = txt
		
	cnt = count(_props)
	repeat with i = 1 to cnt
		do("m."&_props.getPropAt(i)&"=_props[i]")
	end repeat

	m.scriptText = "on mouseDown"&RETURN&callback&RETURN&"end"
	_movie.puppetSprite(_sn, TRUE)
	sprite(_sn).member = m
	sprite(_sn).loc = point(_x, _y)
	_x = _x + w + _dw --8
	_sn = _sn + 1
end

----------------------------------------
--
----------------------------------------
on menuInit me	
	_menuData = [:]
end

----------------------------------------
--
----------------------------------------
on menuInstall (me)
	s = ""
	cnt = _menuData.count
	repeat with i = 1 to cnt
		
		put "menu: "&_menuData.getPropAt(i)&RETURN after s
		m = _menuData[i]
		repeat with tItem in m
			if tItem["name"]="-" then
				put "(-|"&RETURN after s
			else
				if tItem["enabled"]=0 then put "(" after s
				if tItem["checkMark"]=1 then put "! " after s
				put tItem["name"] after s
				if string(tItem["letter"])<>"" then put "/"&tItem["letter"] after s
				put "|"&tItem["handler"]&RETURN after s
			end if
		end repeat
	end repeat
	
	_menuField.text = s
	installMenu _menuField
end

----------------------------------------
--
----------------------------------------
on menuUninstall (me)
	installMenu 0
end
		
----------------------------------------
--
----------------------------------------
on menuAddMenu (me, tMenuName)
	me._menuData.addProp(tMenuName, [])
end

----------------------------------------
--
----------------------------------------
on menuAddItem (me, tMenuName, tItem, tUpdateFlag)
	if voidP(tItem["enabled"]) then tItem["enabled"]=1
	if voidP(tItem["checkMark"]) then tItem["checkMark"]=0
	me._menuData[tMenuName].add(tItem)
	if tUpdateFlag then me.update()
end

----------------------------------------
--
----------------------------------------
on menuAddSep (me, tMenuName)
	me.menuAddItem (tMenuName, ["name": "-"])
end

----------------------------------------
--
----------------------------------------
on menuGetItemEnabledEnabled (me, tMenuName, tItemName)
	tMenu = _menuData[tMenuName]
	repeat with iItem in tMenu
		if iItem["name"]=tItemName then
			return iItem["enabled"]
		end if
	end repeat
end

----------------------------------------
--
----------------------------------------
on menuSetItemEnabledEnabled (me, tMenu, tItemName, flag)
	cnt = _menuData[tMenu].count
	repeat with i = 1 to cnt
		if _menuData[tMenu][i]["name"]=tItemName then
			_menuData[tMenu][i]["enabled"]=flag
			set the enabled of menuItem i of menu tMenu to flag
			exit repeat
		end if
	end repeat
end

----------------------------------------
--
----------------------------------------
on menuGetItemCheckMark (me, tMenuName, tItemName)
	tMenu = _menuData[tMenuName]
	repeat with iItem in tMenu
		if iItem["name"]=tItemName then
			return iItem["checkMark"]
		end if
	end repeat
end

----------------------------------------
--
----------------------------------------
on menuSetItemCheckMark (me, tMenuName, tItemName, flag)
	tMenu = _menuData[tMenuName]
	cnt = tMenu.count
	repeat with i = 1 to cnt
		if tMenu[i]["name"]=tItemName then
			tMenu[i]["checkMark"]=flag
			set the checkMark of menuItem i of menu tMenuName to flag
			exit repeat
		end if
	end repeat
end

----------------------------------------
--
----------------------------------------
on menuTriggerKey (me, tKey)
	repeat with tMenu in _menuData
		repeat with tItem in tMenu
			if tItem["letter"]=tKey then
				do(tItem["handler"])
				exit repeat
			end if
		end repeat
	end repeat
end
